@@ -23,7 +23,6 @@ using namespace std;
 namespace acng
 {
 #define MAXTEMPDELAY acng::cfg::maxtempdelay // 27
-mstring ACNG_API sReplDir("_altStore" SZPATHSEP);
 
 static tFiGlobMap mapItems;
 #ifndef MINIBUILD
@@ -109,7 +108,7 @@ uint64_t fileitem::TakeTransferCount()
 	return ret;
 }
 
-int fileitem::GetFileFd() {
+unique_fd fileitem::GetFileFd() {
 	LOGSTART("fileitem::GetFileFd");
 	setLockGuard;
 
@@ -118,11 +117,11 @@ int fileitem::GetFileFd() {
 
 #ifdef HAVE_FADVISE
 	// optional, experimental
-	if(fd>=0)
+	if(fd != -1)
 		posix_fadvise(fd, 0, m_nSizeChecked, POSIX_FADV_SEQUENTIAL);
 #endif
 
-	return fd;
+	return unique_fd(fd);
 }
 
 off_t GetFileSize(cmstring & path, off_t defret)
@@ -240,6 +239,7 @@ bool fileitem::CheckUsableRange_unlocked(off_t nRangeLastByte)
 			&& atoofft(m_head.h[header::CONTENT_LENGTH], -255) > nRangeLastByte);
 }
 
+// XXX: bForce is ultima ratio and there should be a better way; draft in the revamp branch...
 bool fileitem::SetupClean(bool bForce)
 {
 	setLockGuard;
@@ -699,7 +699,10 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 			if (m_filefd >= 0 && !m_head.h[header::CONTENT_LENGTH])
 			{
 				m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
-				m_head.StoreToFile(CACHE_BASE + m_sPathRel + ".head");
+				// only update the file on disk if this item is still shared
+				lockguard lockGlobalMap(mapItemsMx);
+				if(m_globRef != mapItems.end())
+					m_head.StoreToFile(SABSPATHEX(m_sPathRel, ".head"));
 			}
 		}
 	}
@@ -738,17 +741,11 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 	return true;
 }
 
-fileItemMgmt::~fileItemMgmt()
-{
-	LOGSTART("fileItemMgmt::~fileItemMgmt");
-	reset();
-}
-
-void fileItemMgmt::reset()
+TFileItemUser::~TFileItemUser()
 {
 	LOGSTARTFUNC
 
-	if(!m_ptr) // unregistered before?
+	if (!m_ptr) // unregistered before? or not shared?
 		return;
 
 	lockguard managementLock(mapItemsMx);
@@ -795,10 +792,10 @@ void fileItemMgmt::reset()
 	}
 }
 
-
-bool fileItemMgmt::PrepareRegisteredFileItemWithStorage(cmstring &sPathUnescaped, bool bConsiderAltStore)
+TFileItemUser TFileItemUser::Create(cmstring &sPathUnescaped, bool makeWay)
 {
-	LOGSTARTFUNCx(bConsiderAltStore);
+	LOGSTARTFUNCxs(sPathUnescaped);
+	TFileItemUser ret;
 
 	try
 	{
@@ -807,87 +804,92 @@ bool fileItemMgmt::PrepareRegisteredFileItemWithStorage(cmstring &sPathUnescaped
 		auto it = mapItems.find(sPathRel);
 		if(it != mapItems.end())
 		{
-			if (bConsiderAltStore)
+			auto& fi = it->second;
+			if (makeWay && !fi->m_sPathRel.empty())
 			{
-				// detect items that got stuck somehow
+				// detect items that got stuck somehow and move it out of the way
 				time_t now(GetTime());
-				time_t extime(now - cfg::stucksecs);
-				if (it->second->m_nTimeDlDone < extime)
+				if(fi->m_nTimeDlDone > now + MAXTEMPDELAY  + 2
+						|| fi->m_nTimeDlStarted > now - cfg::stucksecs)
 				{
-					// try to find its sibling which is in good state?
-					for (; it!=mapItems.end() && it->first == sPathRel; ++it)
+
+					auto pathAbs = SABSPATH(fi->m_sPathRel);
+					auto xName = pathAbs + ltos(now);
+					if(0 != creat(xName.c_str(), cfg::fileperms))
 					{
-						if (it->second->m_nTimeDlDone >= extime)
-						{
-							it->second->usercount++;
-							LOG("Sharing an existing REPLACEMENT file item");
-							m_ptr = it->second;
-							return true;
-						}
+						// oh, that's bad, no permissions on the folder whatsoever?
+						log::err(string("Failure to create replacement of ") + fi->m_sPathRel + " - CHECK FOLDER PERMISSIONS!");
+					}
+					else
+					{
+						// if we can create files there then renaming should not be a problem
+						unlink(pathAbs.c_str());
+						rename(xName.c_str(), pathAbs.c_str());
+						fi->m_globRef = mapItems.end();
+						mapItems.erase(it);
+						goto add_as_new;
 					}
-					// ok, then create a modded name version in the replacement directory
-					mstring sPathRelMod(sPathRel);
-					replaceChars(sPathRelMod, "/\\", '_');
-					sPathRelMod.insert(0, sReplDir + ltos(getpid()) + "_" + ltos(now) + "_");
-					LOG("Registering a new REPLACEMENT file item...");
-					auto sp(make_shared<fileitem_with_storage>(sPathRelMod, 1));
-					sp->m_globRef = mapItems.insert(make_pair(sPathRel, sp));
-					m_ptr = sp;
-					return true;
 				}
 			}
 			LOG("Sharing existing file item");
 			it->second->usercount++;
-			m_ptr = it->second;
-			return true;
+			ret.m_ptr = it->second;
+			return ret;
+		}
+		else
+		{
+			add_as_new:
+			LOG("Registering the NEW file item...");
+			auto sp(make_shared<fileitem_with_storage>(sPathRel));
+			sp->usercount++;
+			auto res = mapItems.emplace(sPathRel, sp);
+			ASSERT(res.second);
+			sp->m_globRef = res.first;
+			//lockGlobalMap.unLock();
+			ret.m_ptr = sp;
+			return ret;
 		}
-		LOG("Registering the NEW file item...");
-		auto sp(make_shared<fileitem_with_storage>(sPathRel, 1));
-		sp->m_globRef = mapItems.insert(make_pair(sPathRel, sp));
-		//lockGlobalMap.unLock();
-		m_ptr = sp;
-		return true;
 	}
 	catch(std::bad_alloc&)
 	{
+		return TFileItemUser();
 	}
-	return false;
 }
 
 // make the fileitem globally accessible
-bool fileItemMgmt::RegisterFileItem(tFileItemPtr spCustomFileItem)
+TFileItemUser TFileItemUser::Create(tFileItemPtr spCustomFileItem, bool isShareable)
 {
-	LOGSTARTFUNCx(spCustomFileItem->m_sPathRel);
+	LOGSTARTFUNCxs(spCustomFileItem->m_sPathRel);
+
+	TFileItemUser ret;
 
 	if (!spCustomFileItem || spCustomFileItem->m_sPathRel.empty())
-		return false;
+		return ret;
 
-	reset();
+
+	if(!isShareable)
+	{
+		ret.m_ptr = spCustomFileItem;
+	}
 
 	lockguard lockGlobalMap(mapItemsMx);
 
-	if(ContHas(mapItems, spCustomFileItem->m_sPathRel))
-		return false; // conflict, another agent is already active
-
-	spCustomFileItem->m_globRef = mapItems.emplace(spCustomFileItem->m_sPathRel,
+	auto installed = mapItems.emplace(spCustomFileItem->m_sPathRel,
 			spCustomFileItem);
-	spCustomFileItem->usercount = 1;
-	m_ptr = spCustomFileItem;
-	return true;
-}
 
-void fileItemMgmt::RegisterFileitemLocalOnly(fileitem* replacement)
-{
-	LOGSTARTFUNCx(replacement);
-	reset();
-	m_ptr.reset(replacement);
-}
+	if(!installed.second)
+		return ret; // conflict, another agent is already active
 
+	spCustomFileItem->m_globRef = installed.first;
+	spCustomFileItem->usercount++;
+	ret.m_ptr = spCustomFileItem;
+	return ret;
+}
 
 // this method is supposed to be awaken periodically and detects items with ref count manipulated by
 // the request storm prevention mechanism. Items shall be be dropped after some time if no other
 // thread but us is using them.
-time_t fileItemMgmt::BackgroundCleanup()
+time_t TFileItemUser::BackgroundCleanup()
 {
 	auto now = GetTime();
 	LOGSTARTFUNCsx(now);
@@ -935,6 +937,9 @@ time_t fileItemMgmt::BackgroundCleanup()
 
 ssize_t fileitem_with_storage::SendData(int out_fd, int in_fd, off_t &nSendPos, size_t count)
 {
+	if(out_fd == -1 || in_fd == -1)
+		return -1;
+
 #ifndef HAVE_LINUX_SENDFILE
 	return sendfile_generic(out_fd, in_fd, &nSendPos, count);
 #else
@@ -947,7 +952,7 @@ ssize_t fileitem_with_storage::SendData(int out_fd, int in_fd, off_t &nSendPos,
 #endif
 }
 
-void fileItemMgmt::dump_status()
+void TFileItemUser::dump_status()
 {
 	tSS fmt;
 	log::err("File descriptor table:\n");
@@ -971,7 +976,7 @@ void fileItemMgmt::dump_status()
 					<< item.second->m_nSizeSeen
 					<< "\n\tGotAt: " << item.second->m_nTimeDlStarted << "\n\n";
 		}
-		log::err(fmt.c_str(), nullptr);
+		log::err(fmt);
 	}
 	log::flush();
 }
@@ -979,12 +984,6 @@ void fileItemMgmt::dump_status()
 fileitem_with_storage::~fileitem_with_storage()
 {
 	Truncate2checkedSize();
-
-	if(startsWith(m_sPathRel, sReplDir))
-	{
-		::unlink(SZABSPATH(m_sPathRel));
-		::unlink((SABSPATH(m_sPathRel)+".head").c_str());
-	}
 }
 
 int fileitem_with_storage::Truncate2checkedSize()
@@ -994,7 +993,7 @@ int fileitem_with_storage::Truncate2checkedSize()
 	return ftruncate(m_filefd, m_nSizeChecked);
 }
 
-// special file? When it's rewritten from start, save the old version instead
+// special file? When it's rewritten from start, save the old version aside
 int fileitem_with_storage::MoveRelease2Sidestore()
 {
 	if(m_nSizeChecked)
