@@ -48,13 +48,10 @@ atomic_int nConCount(0), nDisconCount(0), nReuseCount(0);
 namespace acng
 {
 
-ACNG_API std::atomic_uint dl_con_factory::g_nconns(0);
 ACNG_API dl_con_factory g_tcp_con_factory;
 
 tcpconnect::tcpconnect(cfg::tRepoData::IHookHandler *pObserver) : m_pStateObserver(pObserver)
 {
-	if(cfg::maxdlspeed != cfg::RESERVED_DEFVAL)
-		dl_con_factory::g_nconns.fetch_add(1);
 	if(pObserver)
 		pObserver->OnAccess();
 }
@@ -63,8 +60,6 @@ tcpconnect::~tcpconnect()
 {
 	LOGSTART("tcpconnect::~tcpconnect, terminating outgoing connection class");
 	Disconnect();
-	if(cfg::maxdlspeed != cfg::RESERVED_DEFVAL)
-		dl_con_factory::g_nconns.fetch_add(-1);
 #ifdef HAVE_SSL
 	if(m_ctx)
 	{
@@ -79,6 +74,8 @@ tcpconnect::~tcpconnect()
 
 	}
 }
+// UNTESTED blocking of IP protocol when appearing to fail permanently
+#ifdef PROTO_BLOCKER
 
 class tProtoBlocker
 {
@@ -107,7 +104,8 @@ public:
 		auto before = what->fetch_add(fail_count);
 		if(before < threshhold && (before+fail_count) >= threshhold)
 		{
-			event_base_once(evabase::instance->base, -1, EV_TIMEOUT, reset_cb, &what, &block_time);
+			event_base_once(evabase::base, -1, EV_TIMEOUT, reset_cb,
+					&what, &block_time);
 		}
 	}
 	bool can_connect(int proto)
@@ -124,24 +122,27 @@ public:
 
 	}
 } proto_blocker;
+#endif
 
-inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
+std::string tcpconnect::_Connect(int timeout)
 {
 	LOGSTARTFUNCx(m_sHostName, timeout);
 
-	auto dns = CAddrInfo::CachedResolve(m_sHostName, m_sPort);
+	auto dnsres = CAddrInfo::Resolve(m_sHostName, m_sPort);
 
 	unsigned timeouts_v4(0), timeouts_v6(0);
+#ifdef PROTO_BLOCKER
+
 	tDtorEx transfer_timeout_counts([&](){
 		proto_blocker.report(PF_INET, timeouts_v4);
 		proto_blocker.report(PF_INET6, timeouts_v6);
 	});
+#endif
 
-	if(!dns || dns->HasError())
+	if(!dnsres || !dnsres->getTcpAddrInfo())
 	{
-		sErrorMsg = dns->GetError();
-		USRDBG(sErrorMsg);
-		return false; // sErrorMsg got the info already, no other chance to fix it
+		USRDBG(dnsres->getError());
+		return dnsres->getError();
 	}
 
 	::signal(SIGPIPE, SIG_IGN);
@@ -170,7 +171,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 		{
 			checkforceclose(fd);
 			tmexp = time_exp;
-
+#ifdef PROTO_BLOCKER
 			// check if temporary blacklisted
 			if(!proto_blocker.can_connect(dns->ai_family))
 			{
@@ -178,7 +179,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 				state = HANDLE_ERROR;
 				return false;
 			}
-
+#endif
 			fd = ::socket(dns->ai_family, dns->ai_socktype, dns->ai_protocol);
 			if(fd == -1)
 			{
@@ -209,30 +210,27 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 			}
 		}
 	};
-	auto iter = tAlternatingDnsIterator(dns->getTcpAddrInfo());
+	auto iter = tAlternatingDnsIterator(dnsres->getTcpAddrInfo());
 	tConData prim {ADDR_PICKED, -1, time_start + timeout, iter.next() };
 	tConData alt {NO_ALTERNATIVES, -1, time_start + cfg::fasttimeout, nullptr };
 	CTimeVal tv;
 	// pickup the first and/or probably the best errno code which can be reported to user
 	int error_prim = 0;
 
-	auto retGood = [&](tConData& condata)
-			{
-		LOG(condata.fd);
-		std::swap(condata.fd, m_conFd);
+	auto retGood = [&](int &fd)
+	{
+		std::swap(fd, m_conFd);
+
+#ifdef PROTO_BLOCKER
 		// mark this as good protocol, report failure count on other on return later
-		if(condata.dns->ai_family == PF_INET6)
+			if(condata.dns->ai_family == PF_INET6)
 			timeouts_v6 = 0;
-		else
+			else
 			timeouts_v4 = 0;
-		return true;
+#endif
+			return sEmptyString;
 	};
-	auto retError = [&](const std::string &errStr) {
-		LOG(errStr);
-		sErrorMsg = errStr;
-		return false;
-	};
-	auto withErrnoError = [&]() { return retError(tErrnoFmter("500 Connection failure: "));	};
+	auto withErrnoError = [&]() { return tErrnoFmter("500 Connection failure: ");	};
 	auto withThisErrno = [&withErrnoError](int myErr) { errno = myErr; return withErrnoError(); };
 
 	// ok, initial condition, one target should be always there, iterator would also hop to the next fallback if allowed
@@ -240,7 +238,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 	{
 		dbgline;
 		if(cfg::conprotos[0] == PF_UNSPEC)
-			return retError("523 Target address could not be resolved");
+			return "523 No such host";
 		else
 			return withThisErrno(EAFNOSUPPORT);
 	}
@@ -265,7 +263,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 			break;
 		case ADDR_PICKED:
 			if(prim.init_con(time_start + cfg::nettimeout))
-				return retGood(prim);
+				return retGood(prim.fd);
 			OPTSET(error_prim, errno);
 			__just_fall_through;
 		case SELECT_CONN:
@@ -305,7 +303,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 			checkforceclose(prim.fd);
 			break;
 		default: // this should be unreachable
-			return retError("500 Internal error at " STRINGIFY(__LINE__));
+			return "500 Internal error at " STRINGIFY(__LINE__);
 		}
 
 		switch(alt.state)
@@ -322,7 +320,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 		case ADDR_PICKED:
 		{
 			if(alt.init_con(GetTime() + cfg::fasttimeout))
-				return retGood(alt);
+				return retGood(alt.fd);
 			continue;
 		}
 		case PICK_ADDR:
@@ -367,7 +365,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 			break;
 		}
 		default: // this should be unreachable
-			return retError("500 Internal error at " STRINGIFY(__LINE__));
+			return "500 Internal error at " STRINGIFY(__LINE__);
 		}
 
 		select_set_t selset;
@@ -406,7 +404,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 						prim.state = HANDLE_ERROR;
 					}
 					else
-						return retGood(*p);
+						return retGood(p->fd);
 				}
 			}
 		}
@@ -450,7 +448,7 @@ ACNG_API void CloseAllCachedConnections()
 
 tDlStreamHandle dl_con_factory::CreateConnected(cmstring &sHostname, cmstring &sPort,
 		mstring &sErrOut, bool *pbSecondHand, cfg::tRepoData::IHookHandler *pStateTracker
-		,bool bSsl, int timeout, bool nocache)
+		,bool bSsl, int timeout, bool nocache) const
 {
 	LOGSTARTFUNCsx(sHostname, sPort, bSsl);
 
@@ -508,7 +506,7 @@ tDlStreamHandle dl_con_factory::CreateConnected(cmstring &sHostname, cmstring &s
 			p->m_sPort=sPort;
 		}
 
-		if(!p || !p->_Connect(sErrOut, timeout) || p->GetFD()<0) // failed or worthless
+		if(!p || !p->_Connect(timeout).empty() || p->GetFD()<0) // failed or worthless
 			p.reset();
 #ifdef HAVE_SSL
 		else if(bSsl)
@@ -528,12 +526,12 @@ tDlStreamHandle dl_con_factory::CreateConnected(cmstring &sHostname, cmstring &s
 	return p;
 }
 
-void dl_con_factory::RecycleIdleConnection(tDlStreamHandle & handle)
+void dl_con_factory::RecycleIdleConnection(tDlStreamHandle & handle) const
 {
 	if(!handle)
 		return;
 
-	LOGSTARTFUNCsx(handle->m_sHostName);
+	LOGSTARTFUNCxs(handle->m_sHostName);
 
 	if(handle->m_pStateObserver)
 	{
@@ -659,19 +657,23 @@ bool tcpconnect::SSLinit(mstring &sErr, cmstring &sHostname, cmstring &sPort)
 	SSL * ssl(nullptr);
 	mstring ebuf;
 
-	auto withSslError = [&sErr](const char *perr)
+	auto withSslHeadPfx = [&sErr](const char *perr)
 					{
 		sErr="500 SSL error: ";
 		sErr+=(perr?perr:"Generic SSL failure");
 		return false;
 					};
-	auto withLastSslError = [&withSslError]()
+	auto withLastSslError = [&withSslHeadPfx]()
 						{
-		return withSslError(ERR_reason_error_string(ERR_get_error()));
+		auto nErr = ERR_get_error();
+		auto serr = ERR_reason_error_string(nErr);
+		return withSslHeadPfx(serr);
 						};
-	auto withRetCode = [&withSslError, &ssl](int hret)
+	auto withSslRetCode = [&withSslHeadPfx, &withLastSslError, &ssl](int hret)
 				{
-		return withSslError(ERR_reason_error_string(SSL_get_error(ssl, hret)));
+		auto nErr = SSL_get_error(ssl, hret);
+		auto serr =  ERR_reason_error_string(nErr);
+		return serr ? withSslHeadPfx(serr) : withLastSslError();
 				};
 
 	// cleaned up in the destructor on EOL
@@ -707,45 +709,52 @@ bool tcpconnect::SSLinit(mstring &sErr, cmstring &sHostname, cmstring &sPort)
  			| SSL_MODE_ENABLE_PARTIAL_WRITE);
 
  	auto hret=SSL_set_fd(ssl, m_conFd);
- 	if(hret != 1) return withRetCode(hret);
+ 	if(hret != 1) return withSslRetCode(hret);
 
  	while(true)
  	{
  		hret=SSL_connect(ssl);
- 		if(hret == 1 )
- 			break;
- 		if(hret == 0)
- 			return withRetCode(hret);
+		if (hret == 1)
+			break;
 
-		fd_set rfds, wfds;
-		FD_ZERO(&rfds);
-		FD_ZERO(&wfds);
- 		switch(SSL_get_error(ssl, hret))
- 		{
- 		case SSL_ERROR_WANT_READ:
- 			FD_SET(m_conFd, &rfds);
- 			break;
- 		case SSL_ERROR_WANT_WRITE:
- 			FD_SET(m_conFd, &wfds);
- 			break;
- 		default:
- 			return withRetCode(hret);
- 		}
-		int nReady=select(m_conFd+1, &rfds, &wfds, nullptr, CTimeVal().ForNetTimeout());
-		if(!nReady) return withSslError("Socket timeout");
-		if (nReady<0)
+		if (hret == 0)
+			return withSslRetCode(hret);
+
+		if (hret == -1)
 		{
-#ifndef MINIBUILD
-			ebuf=tErrnoFmter("Socket error");
-			return withSslError(ebuf.c_str());
-#else
-			return withSslError("Socket error");
-#endif
+			auto nError = SSL_get_error(ssl, hret);
+			fd_set rfds, wfds;
+			FD_ZERO(&rfds);
+			FD_ZERO(&wfds);
+			switch (nError)
+			{
+			case SSL_ERROR_WANT_READ:
+				FD_SET(m_conFd, &rfds);
+				break;
+			case SSL_ERROR_WANT_WRITE:
+				FD_SET(m_conFd, &wfds);
+				break;
+			default:
+				return withSslRetCode(nError);
+			}
+			int nReady = select(m_conFd + 1, &rfds, &wfds, nullptr, CTimeVal().ForNetTimeout());
+			if (nReady == 1)
+				continue;
+			if (nReady == 0)
+				return withSslHeadPfx("Socket timeout");
+			if (nReady < 0)
+			{
+				ebuf = tErrnoFmter("Socket error");
+				return withSslHeadPfx(ebuf.c_str());
+			}
 		}
+		else
+			return withLastSslError();
+
  	}
  	if(m_bio) BIO_free_all(m_bio);
  	m_bio = BIO_new(BIO_f_ssl());
- 	if(!m_bio) return withSslError("IO initialization error");
+ 	if(!m_bio) return withSslHeadPfx("IO initialization error");
  	// not sure we need it but maybe the handshake can access this data
  	BIO_set_conn_hostname(m_bio, sHostname.c_str());
  	BIO_set_conn_port(m_bio, sPort.c_str());
@@ -760,7 +769,7 @@ bool tcpconnect::SSLinit(mstring &sErr, cmstring &sHostname, cmstring &sPort)
 		X509* server_cert = nullptr;
 		hret=SSL_get_verify_result(ssl);
 		if( hret != X509_V_OK)
-			return withSslError(X509_verify_cert_error_string(hret));
+			return withSslHeadPfx(X509_verify_cert_error_string(hret));
 		server_cert = SSL_get_peer_certificate(ssl);
 		if(server_cert)
 		{
@@ -769,26 +778,11 @@ bool tcpconnect::SSLinit(mstring &sErr, cmstring &sHostname, cmstring &sPort)
 			X509_free(server_cert);
 		}
 		else // The handshake was successful although the server did not provide a certificate
-			return withSslError("Incompatible remote certificate");
+			return withSslHeadPfx("Incompatible remote certificate");
 	}
 	return true;
 }
 
-//! Global initialization helper (might be non-reentrant)
-void ACNG_API globalSslInit()
-{
-	static bool inited=false;
-	if(inited)
-		return;
-	inited = true;
-	SSL_load_error_strings();
-	ERR_load_BIO_strings();
-	ERR_load_crypto_strings();
-	ERR_load_SSL_strings();
-	OpenSSL_add_all_algorithms();
-	SSL_library_init();
-}
-
 #endif
 
 bool tcpconnect::StartTunnel(const tHttpUrl& realTarget, mstring& sError,
